/*!\file window.c 
 * \brief Exemple d'clairage avec dessin de sphere.
 * \author : Fars BELHADJ (Lire COPYING)  amsi@ai.univ-paris8.fr
 * \date  22/10/2007
 */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#if defined(__APPLE__) || defined(MACOSX)
# include <GLUT/glut.h> /* Ils auraient pu m'eviter ca ! */
#else
# include <GL/glut.h>
#endif

static        void glutInitGL   (void);
static        void glutReshape  (int l, int h);
static        void glutDisplay  (void);
static        void glutIdle     (void);
static        void glutSpecialKeyboard(int k, int x, int y);
static        void glutKeyboard (unsigned char k, int x, int y);
static        void sphere(int);
static inline void glutFPS(void);


static int pause = 0, longiMax = 20, latiMax = 10;

int main(int argc, char ** argv) {
  int w = 640, h = 480;
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(w, h);
  glutCreateWindow("are you there ?");
  glutInitGL();
  glutDisplayFunc(glutDisplay);
  glutIdleFunc(glutIdle);
  glutReshapeFunc(glutReshape);
  glutSpecialFunc(glutSpecialKeyboard);
  glutKeyboardFunc(glutKeyboard);
  glutMainLoop();
  sphere(1);
  return 0;
}

/* 
 * glutInitGL : configurer les parametres GL.
 */
static void glutInitGL(void) {
  glShadeModel(GL_SMOOTH);
  glDepthFunc(GL_LEQUAL);
  glFrontFace(GL_CCW);
  glCullFace(GL_BACK);
  glClearColor(0, 0, 0, 0);
  glClearDepth(1.0);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glDisable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);

  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glDisable(GL_LIGHTING);
  glDisable(GL_LIGHT0);
}

/* 
 * glutReshape : modifier la matrice de projection selon le
 * redimensionnement de la fenetre glut.
 */
static void glutReshape(int l, int h) {
  glViewport(0, 0, l, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, l / (float)h, 0.1, 100.0);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity( );
}

/*
 * glutIdle : fonction appelee a chaque tour de boucle dans
 * glutLoop().
 */
static void glutIdle(void) {
  glutFPS();
  glutPostRedisplay();
}


static void glutSpecialKeyboard(int k, int x, int y) {
  switch (k) {
  case GLUT_KEY_UP:
    latiMax++;
    goto etiquette;
  case GLUT_KEY_DOWN:
    latiMax--;
    goto etiquette;
  case GLUT_KEY_RIGHT:
    longiMax++;
    goto etiquette;
  case GLUT_KEY_LEFT:
    longiMax--;
  etiquette:
    if(longiMax < 3)
      longiMax = 3;
    else if(latiMax < 3)
      latiMax = 3;
    else
      sphere(1);
    break;
  }
}

static void glutKeyboard(unsigned char k, int x, int y) {
  GLint v[2];
  switch(k) {
  case ' ':
  case 'p':
    pause = !pause;
    break;
  case 'w':
    glGetIntegerv(GL_POLYGON_MODE, v);
    if(v[0] == GL_FILL)
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    else
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    break;
  case 'f':
    glGetIntegerv(GL_SHADE_MODEL, v);
    if(v[0] == GL_FLAT)
      glShadeModel(GL_SMOOTH);
    else
      glShadeModel(GL_FLAT);
    break;
  case 'l':
    if(glIsEnabled(GL_LIGHTING)) {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
    } else {
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
    }
    break;
  case 27:
  case 'q':
    exit(0);
  default:
    break;
  }
}

/*
 * glutDisplay : calculer une rotation puis une translation dans le repere
 * MODELVIEW puis dessiner le triangle colore. incrementer l'angle de rotation.
 */
static void glutDisplay(void) {
  GLfloat dt;
  static GLfloat a = 0.0;
  static int t0 = 0, t1;
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  /* rcupration du dt */
  t1 = glutGet(GLUT_ELAPSED_TIME);
  dt = (t1 - t0) / 1000.0;
  t0 = t1;
  if(!pause)
    a += 90 * dt;
  /* fin de rcupration du dt */
  glLoadIdentity();
  glTranslatef(0.0f,0.0f,-3.5f);
  /* regarder d'un peu plus haut */
  glRotatef(30, 1, 0, 0);
  glRotatef(a, 0, 1, 0);
  sphere(0);
  glutSwapBuffers();
}

/*\!brief dessine une sphre unitaire colorie ou ombre (vecteurs
 * normaux) ; cette fonction utilise des QUADS.
 *
 * \param onlyFreeData demande  la fonction de librer sa donne sans
 * rien mettre  GL.
 */
static void sphere(int onlyFreeData) {
  int i, e;
  static GLfloat * vertices = NULL;
  GLfloat phi0, phi1, theta0, theta1;
  if(onlyFreeData) {
    if(vertices) {
      free(vertices);
      vertices = NULL;
    }
    return;
  }
  if(!vertices) {
    vertices = malloc(2 * latiMax * 2 * longiMax * 3 * sizeof *vertices);
    assert(vertices);
  }
  for(e = 0 ; e < latiMax; e++) {
    theta0 = -(M_PI / 2) + (e * M_PI) / (GLfloat)(latiMax);
    theta1 = -(M_PI / 2) + ((e + 1) * M_PI) / (GLfloat)(latiMax);
    for(i = 0; i < longiMax; i++) {
      phi0 = (2.0 * M_PI * i) / (GLfloat)longiMax;
      phi1 = (2.0 * M_PI * (i + 1)) / (GLfloat)longiMax;
      vertices[12 * (longiMax * e + i) + 0] = cos(theta0) * cos(phi0);
      vertices[12 * (longiMax * e + i) + 1] = sin(theta0);
      vertices[12 * (longiMax * e + i) + 2] = cos(theta0) * sin(phi0);
      vertices[12 * (longiMax * e + i) + 3] = cos(theta0) * cos(phi1);
      vertices[12 * (longiMax * e + i) + 4] = sin(theta0);
      vertices[12 * (longiMax * e + i) + 5] = cos(theta0) * sin(phi1);
      vertices[12 * (longiMax * e + i) + 6] = cos(theta1) * cos(phi1);
      vertices[12 * (longiMax * e + i) + 7] = sin(theta1);
      vertices[12 * (longiMax * e + i) + 8] = cos(theta1) * sin(phi1);
      vertices[12 * (longiMax * e + i) + 9] = cos(theta1) * cos(phi0);
      vertices[12 * (longiMax * e + i) + 10] = sin(theta1);
      vertices[12 * (longiMax * e + i) + 11] = cos(theta1) * sin(phi0);
    }
  }
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_COLOR_ARRAY);
  glVertexPointer(3, GL_FLOAT, 0, vertices);
  glNormalPointer(GL_FLOAT, 0, vertices);
  glColorPointer(3, GL_FLOAT, 0, vertices);
  glDrawArrays(GL_QUADS, 0, 2 * 2 * longiMax * latiMax);
  glDisableClientState(GL_COLOR_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
}

/*
 * glutFPS : compter le nombre de frames affiches. imprimer le
 * framerate toutes les 5 secondes.
 */
static inline void glutFPS(void) {
  unsigned int t;
  static unsigned int t0 = 0, f = 0;
  f++;
  t = glutGet(GLUT_ELAPSED_TIME);
  if(t - t0 > 5000) {
    fprintf(stderr, "glutFPS:%6d\n",
            (int)(1000.0 * f / (t - t0)));
    t0 = t;
    f  = 0;
  }
}
